@@ -13,24 +13,16 @@ from pymatgen.core.structure import Structure, Molecule
 
 # PyXtal imports #avoid *
 from pyxtal.version import __version__
-from pyxtal.molecular_crystal import (
-    molecular_crystal,
-    molecular_crystal_2D,
-    molecular_crystal_1D,
-)
-from pyxtal.crystal import (
-    random_cluster,
-    random_crystal,
-    random_crystal_1D,
-    random_crystal_2D,
-)
+from pyxtal.block_crystal import block_crystal
+from pyxtal.crystal import random_crystal
 from pyxtal.symmetry import Group, Wyckoff_position, search_matched_position
 from pyxtal.operations import apply_ops, SymmOp, get_inverse
-from pyxtal.wyckoff_site import atom_site, mol_site, WP_merge
+from pyxtal.wyckoff_site import atom_site, mol_site
 from pyxtal.wyckoff_split import wyckoff_split
 from pyxtal.molecule import pyxtal_molecule
 from pyxtal.lattice import Lattice
 from pyxtal.tolerance import Tol_matrix
+from pyxtal.representation import representation
 from pyxtal.io import read_cif, write_cif, structure_from_ext
 from pyxtal.XRD import XRD
 from pyxtal.constants import letters
@@ -158,7 +150,7 @@ class pyxtal:
             s = "\n------Crystal from {:s}------".format(self.source)
             s += "\nDimension: {}".format(self.dim)
             s += "\nComposition: {}".format(self.formula)
-            if self.group.number in [5, 7, 8, 9, 12, 13, 14, 15] and self.diag:
+            if self.diag and self.group.number in [5, 7, 8, 9, 12, 13, 14, 15]:
                 symbol = self.group.alias
             else:
                 symbol = self.group.symbol
@@ -230,6 +222,9 @@ class pyxtal:
         max_count = 10,
         torsions = None,
         force_pass = False,
+        block = None,
+        num_block = None,
+        seed = None,
     ):
         if self.molecular:
             prototype = "molecular"
@@ -243,30 +238,35 @@ class pyxtal:
         while True:
             count += 1
             if self.molecular:
-                if dim == 3:
-                    if numIons is None:
-                        numIons = [len(Group(group)[0])]*len(species)
-                    struc = molecular_crystal(group, species, numIons, factor, lattice=lattice, 
-                    torsions=torsions, sites=sites, conventional=conventional, diag=diag, tm=tm)
-                elif dim == 2:
-                    struc = molecular_crystal_2D(group, species, numIons, factor,
-                    thickness=thickness, sites=sites, conventional=conventional, tm=tm)
-                elif dim == 1:
-                    struc = molecular_crystal_1D(group, species, numIons, factor,
-                    area=area, sites=sites, conventional=conventional, tm=tm)
+                struc = block_crystal(dim,
+                                      group, 
+                                      species, 
+                                      numIons, 
+                                      factor, 
+                                      thickness = thickness,
+                                      area = area,
+                                      block = block,
+                                      num_block = num_block,
+                                      lattice = lattice, 
+                                      torsions = torsions, 
+                                      sites = sites, 
+                                      conventional = conventional, 
+                                      diag = diag, 
+                                      tm = tm,
+                                      seed = seed,
+                                      )
             else:
-                if dim == 3:
-                    struc = random_crystal(group, species, numIons, factor,
-                            lattice, sites, conventional, tm)
-                elif dim == 2:
-                    struc = random_crystal_2D(group, species, numIons, factor,
-                            thickness, lattice, sites, conventional, tm)
-                elif dim == 1:
-                    struc = random_crystal_1D(group, species, numIons, factor,
-                            area, lattice, sites, conventional, tm)
-                else:
-                    struc = random_cluster(group, species, numIons, factor,
-                            lattice, sites, tm)
+                struc = random_crystal(dim, 
+                                       group, 
+                                       species, 
+                                       numIons, 
+                                       factor, 
+                                       thickness, 
+                                       area, 
+                                       lattice, 
+                                       sites, 
+                                       conventional, 
+                                       tm)
             if force_pass:
                 quit = True
                 break
@@ -275,7 +275,7 @@ class pyxtal:
                 break
 
             if count >= max_count:
-                raise RuntimeError("It takes long time to generate the structure, check inputs")
+                raise RuntimeError("long time to generate structure, check inputs")
 
         if quit:
             self.valid = struc.valid
@@ -300,7 +300,7 @@ class pyxtal:
                 pass
 
 
-    def from_seed(self, seed, molecules=None, tol=1e-4, a_tol=5.0, relax_H=False, ignore_HH=True, backend='pymatgen'):
+    def from_seed(self, seed, molecules=None, tol=1e-4, a_tol=5.0, ignore_HH=True, add_H=False, backend='pymatgen'):
         """
         Load the seed structure from Pymatgen/ASE/POSCAR/CIFs
         Internally they will be handled by Pymatgen
@@ -309,17 +309,17 @@ class pyxtal:
             seed: cif/poscar file or a Pymatgen Structure object
             molecules: a list of reference molecule (xyz file or Pyxtal molecule)
             tol: scale factor for covalent bond distance
-            relax_H: whether or not relax the position for hydrogen atoms in structure
             ignore_HH: whether or not ignore the short H-H distance when checking molecule
+            add_H: whether or not add H atoms
         
         """
 
         if self.molecular:
             pmols = []
             for mol in molecules:
-                pmols.append(pyxtal_molecule(mol, fix=True)) #.mol
+                pmols.append(pyxtal_molecule(mol, fix=True))
             #QZ: the default choice will not work for molecular H2, which should be rare!
-            struc = structure_from_ext(seed, pmols, relax_H=relax_H, ignore_HH=ignore_HH)
+            struc = structure_from_ext(seed, pmols, ignore_HH=ignore_HH, add_H=add_H)
             self.mol_sites = struc.make_mol_sites()
             self.group = Group(struc.wyc.number)
             self.lattice = struc.lattice
@@ -918,7 +918,18 @@ class pyxtal:
         mult = len(self.group[0])
         comp = [c/mult for c in self.numMols]
         if integer:
-            comp = [int(c) for c in comp]
+            comp = [int(np.ceil(c)) for c in comp]
+        return comp
+
+    def get_1D_comp(self):
+        """
+        get composition for 1d rep of molecular xtal
+        """
+        comp = [0] * len(self.molecules)
+        for s in self.mol_sites:
+            for i, m in enumerate(self.molecules): 
+                if s.molecule.name == m.name:
+                    comp[i] += 1
         return comp
 
     def get_num_torsions(self):
@@ -980,6 +991,38 @@ class pyxtal:
         else:
             raise RuntimeError("No valid structure can be converted to pymatgen.")
 
+    def to_pyxtal_center(self):
+        """
+        export to PyXtal object for molecular centers only.
+        """
+
+        if self.valid and self.molecular:
+            new_struc = pyxtal()
+            new_struc.lattice = self.lattice.copy()
+            newsites = []
+            for site in self.mol_sites:
+                for i, mol in enumerate(self.molecules):
+                    if mol.name == site.molecule.name:
+                        break
+                newsites.append(atom_site(site.wp, site.position, i+1, diag=site.diag))
+            new_struc.atom_sites = newsites
+            new_struc.group = self.group
+            new_struc.diag = site.diag
+            new_struc.numIons = self.numMols
+            new_struc.species = []
+            for i in range(len(self.molecules)):
+                new_struc.species.append(i+1)
+            new_struc.valid = True
+            new_struc.factor = 1.0
+            new_struc.source = 'Mol. Center'
+            new_struc.dim = self.dim
+            new_struc.PBC = self.PBC
+            new_struc._get_formula()
+            return new_struc
+        else:
+            raise RuntimeError("No valid structure can be converted to pymatgen.")
+
+
     def get_XRD(self, **kwargs):
         """
         compute the PXRD object.
@@ -1002,7 +1045,7 @@ class pyxtal:
         else:
             return display_atomic(self, **kwargs)
 
-    def optimize_lattice(self, iterations=3, force=False):
+    def optimize_lattice(self, iterations=5, force=False):
         """
         optimize the lattice if the cell has a bad inclination angles
         """
@@ -1022,6 +1065,12 @@ class pyxtal:
         """
         pass
 
+    def get_1D_representation(self):
+        """
+        Get the 1D representation class for molecular crystals
+        """
+        return representation.from_pyxtal(self)
+
     def transform(self, trans, lattice=None):
         """
         perform cell transformation which may lead to the change of symmetry operation
@@ -1323,8 +1372,10 @@ class pyxtal:
             self.transform([[1,0,0],[0,1,0],[1,0,1]])
 
         #QZ: below is a tentative solution
-        if self.group.number == 64 and s.mol_sites[0].wp.multiplicity==4:
+        if self.group.number == 64 and self.mol_sites[0].wp.multiplicity==4:
             s = self.subgroup_once(eps=0, mut_lat=False, H=61, group_type='k', ignore_special=True)
+        elif self.group.number == 113 and self.mol_sites[0].wp.multiplicity==2:
+            s = self.subgroup_once(eps=0, mut_lat=False, H=18, group_type='t', ignore_special=True)
         else:
             s = deepcopy(self)
 
