@@ -5,7 +5,7 @@ import numpy as np
 from pymatgen.core.structure import Structure, Molecule
 from pymatgen.core.bonds import CovalentBond
 from pymatgen.symmetry.analyzer import SpacegroupAnalyzer
-from pyxtal.wyckoff_site import atom_site, mol_site, WP_merge
+from pyxtal.wyckoff_site import atom_site, mol_site
 from pyxtal.molecule import pyxtal_molecule, Orientation, compare_mol_connectivity
 from pyxtal.symmetry import Wyckoff_position, Group
 from pyxtal.lattice import Lattice
@@ -26,9 +26,9 @@ def write_cif(struc, filename=None, header="", permission='w', sym_num=None, sty
         struc: pyxtal structure object
         filename: path of the structure file 
         header: additional information
-        permission: write('w') or append('a+') to the given file
+        permission: write(`w`) or append(`a+`) to the given file
         sym_num: the number of symmetry operations, None means writing all symops
-        style: "icsd" or "mp" (used in pymatgen)
+        style: `icsd` or `mp` (used in pymatgen)
 
     """
     if sym_num is None:
@@ -192,14 +192,14 @@ def read_cif(filename):
     lattice = Lattice.from_para(a, b, c, alpha, beta, gamma, lat_type)
     sites = []
     for specie, coord in zip(species, coords):
-        pt, wp, _ = WP_merge(coord, lattice.matrix, wp0, tol=0.1)
+        pt, wp, _ = wp0.merge(coord, lattice.matrix, tol=0.1)
         sites.append(atom_site(wp, pt, specie, diag))
     return lattice, sites
 
 
 class structure_from_ext():
     
-    def __init__(self, struc, ref_mols, tol=0.2, relax_H=False, ignore_HH=False):
+    def __init__(self, struc, ref_mols, tol=0.2, ignore_HH=False, add_H=False):
 
         """
         extract the mol_site information from the give cif file 
@@ -209,9 +209,8 @@ class structure_from_ext():
             struc: cif/poscar file or a Pymatgen Structure object
             ref_mols: a list of reference molecule (xyz file or Pyxtal molecule)
             tol: scale factor for covalent bond distance
-            relax_H: whether or not relax the position for hydrogen atoms in structure
             ignore_HH: whether or not ignore the short H-H distance when checking molecule
-        
+            add_H: whether or not add the H atoms
         """
 
         for ref_mol in ref_mols:
@@ -231,10 +230,13 @@ class structure_from_ext():
             print(type(struc))
             raise NameError("input structure cannot be intepretted")
 
+        # reset the hydrogen position
+        if add_H: pmg_struc.remove_species('H')
+
         self.ref_mols = ref_mols
         self.tol = tol
         self.diag = False
-        self.relax_H = relax_H
+        self.add_H = add_H
 
         sym_struc, number = get_symmetrized_pmg(pmg_struc)
         group = Group(number)
@@ -243,7 +245,7 @@ class structure_from_ext():
         self.perm = [0,1,2]
 
         molecules = search_molecules_in_crystal(sym_struc, self.tol, ignore_HH=ignore_HH)
-        if self.relax_H: molecules = self.addh(molecules)
+
         self.pmg_struc = sym_struc
         self.lattice = Lattice.from_matrix(sym_struc.lattice.matrix, ltype=group.lattice_type)
         self.resort(molecules)
@@ -272,47 +274,48 @@ class structure_from_ext():
                 mults.append(len(tmp_ids))
                 #print("check", id, tmp_ids, mults)
 
-        # add position and molecule
-        # print("ids", ids, mults)
+        # add position and molecule, print("ids", ids, mults)
         N_sites = len(ids)
         self.numMols = [0] * len(self.ref_mols)
         self.positions = []
         self.p_mols = []
         self.wps = []
         ids_done = []
-        for j, mol2 in enumerate(self.ref_mols):
+        for j, mol2_ref in enumerate(self.ref_mols):
+            mol2 = mol2_ref.copy()
+            if self.add_H: mol2.mol.remove_species("H")
+
             for i, id in enumerate(ids):
                 mol1 = molecules[id]
                 if id not in ids_done and len(mol2.mol) == len(mol1):
+                    p_mol = mol2_ref.copy() # create p_mol
                     match, mapping = compare_mol_connectivity(mol2.mol, mol1)
                     if match:
-                        #print(self.numMols)
-                        self.numMols[j] += mults[i]
-                        #print(j, mults[i], self.numMols)
-                        # rearrange the order
-                        order = [mapping[at] for at in range(len(mol1))]
-                        xyz = mol1.cart_coords[order] 
-                        frac = np.dot(xyz, inv_lat) 
-                        xyz = np.dot(frac, new_lat) 
-                        # create p_mol
-                        p_mol = mol2.copy() 
-                        center = p_mol.get_center(xyz) 
-                        #print(xyz-center)
-                        p_mol.reset_positions(xyz-center)
-
-                        position = np.dot(center, np.linalg.inv(new_lat))
+                        self.numMols[j] += mults[i] 
+                        if len(mol1) > 1:
+                            # rearrange the order
+                            order = [mapping[at] for at in range(len(mol1))]
+                            xyz = mol1.cart_coords[order] 
+                            # add hydrogen positions here
+                            if self.add_H: xyz = self.add_Hydrogens(mol2.smile, xyz)
+                            #print(xyz)
+                            frac = np.dot(xyz, inv_lat) 
+                            xyz = np.dot(frac, new_lat) 
+                            center = p_mol.get_center(xyz) 
+                            p_mol.reset_positions(xyz-center)
+                            position = np.dot(center, np.linalg.inv(new_lat))
+                        else:
+                            position = np.dot(mol1.cart_coords[0], np.linalg.inv(new_lat))
                         position -= np.floor(position)
-                        #print(position)
-                        #print(lat)
+                        #print(position); print(lat)
                         #print(p_mol.mol.cart_coords[:10] + np.dot(position, new_lat))
-                        # print(len(self.pmg_struc), len(self.molecule), len(self.wyc))
 
                         # check if molecule is on the special wyckoff position
                         if mults[i] < len(self.wyc):
                             #Transform it to the conventional representation
                             if self.diag: position = np.dot(self.perm, position).T
                             #print("molecule is on the special wyckoff position")
-                            position, wp, _ = WP_merge(position, new_lat, self.wyc, 0.1)
+                            position, wp, _ = self.wyc.merge(position, new_lat, 0.1)
                             self.wps.append(wp)
                             self.numMols[j] = len(wp)
                             #print("After Merge:---"); print(position); print(wp)
@@ -330,17 +333,40 @@ class structure_from_ext():
                     msg += molecules[id].to('xyz')
                     raise ReadSeedError(msg)
 
-    def addh(self, molecules):
+    def add_Hydrogens(self, smile, xyz):
         """
-        add hydrogen
+        add hydrogen for pymtagen molecule
         """
-        from pymatgen.io.babel import BabelMolAdaptor
-        for mol in molecules:
-            ad = BabelMolAdaptor(mol)
-            ad.add_hydrogen()        
-            ad.localopt()
-            mol = ad.pymatgen_mol
-        return molecules
+        from rdkit import Chem
+        from rdkit.Chem import AllChem
+        from rdkit.Geometry import Point3D
+
+        #print(xyz)
+        m1 = Chem.MolFromSmiles(smile)
+        m2 = Chem.AddHs(m1)
+        if len(smile) > 100:
+            AllChem.EmbedMolecule(m2,randomSeed=3)
+        else:
+            AllChem.EmbedMolecule(m2,randomSeed=0xf00d)
+        m2 = Chem.RemoveHs(m2)
+        conf = m2.GetConformer(0)
+
+        for i in range(conf.GetNumAtoms()):
+            x, y, z = xyz[i]
+            conf.SetAtomPosition(i, Point3D(x,y,z))
+        #Chem.MolToPDBFile(m2, 'test.pdb')
+        #conf = m2.GetConformer(0); print(conf.GetPositions())
+
+        m1 = Chem.AddHs(m1)
+        if len(smile) > 100:
+            AllChem.EmbedMolecule(m1, randomSeed=3)
+        else:
+            AllChem.EmbedMolecule(m1, randomSeed=0xf00d)
+        AllChem.UFFOptimizeMolecule(m1)
+        m3 = AllChem.ConstrainedEmbed(m1,m2)
+        conf = m3.GetConformer(0) #; print(conf.GetPositions())
+        
+        return conf.GetPositions()
 
     def make_mol_sites(self):
         """
@@ -410,36 +436,50 @@ def search_molecules_in_crystal(struc, tol=0.2, once=False, ignore_HH=True):
         new_members = []
         for site0 in sites0:
             sites_add, visited = check_one_site(struc, site0, visited)
-            #print(sites_add)
             new_members.extend(sites_add)
         return new_members, visited
     
-    def check_one_site(struc, site0, visited, rmax=2.9):
+    def check_one_site(struc, site0, visited, rmax=2.8):
         neigh_sites = struc.get_neighbors(site0, rmax)
         ids = [m.index for m in visited]
         sites_add = []
         ids_add = []
+        pbc = isinstance(struc, Structure)
 
         for site1 in neigh_sites:
             if (site1.index not in ids+ids_add):
-                if CovalentBond.is_bonded(site0, site1, tol):
-                    (d, image) = site0.distance_and_image(site1)
-                    #QZ: use our own bond distance lib
-                    key = "{:s}-{:s}".format(site1.specie.value, site0.specie.value)
+                try:
+                    if CovalentBond.is_bonded(site0, site1, tol):
+                        if pbc:
+                            (d, image) = site0.distance_and_image(site1)
+                        else:
+                            d = site0.distance(site1)
+                        key = "{:s}-{:s}".format(site1.specie.value, site0.specie.value)
 
-                    #sometime the H-H short distance is not avoidable
-                    if key == 'H-H': 
-                        if not ignore_HH:
-                            site1.frac_coords += image
-                            sites_add.append(site1)
-                            ids_add.append(site1.index)
+                        #sometime the H-H short distance is not avoidable
+                        if key == 'H-H': 
+                            if not ignore_HH:
+                                if pbc: site1.frac_coords += image
+                                sites_add.append(site1)
+                                ids_add.append(site1.index)
+                        else:
+                            if d < bonds[key]:
+                                if pbc: site1.frac_coords += image
+                                sites_add.append(site1)
+                                ids_add.append(site1.index)
+                except ValueError:
+                    #QZ: use our own bond distance lib
+                    if pbc:
+                        (d, image) = site0.distance_and_image(site1)
                     else:
-                        if d < bonds[key]:
-                            site1.frac_coords += image
-                            sites_add.append(site1)
-                            ids_add.append(site1.index)
-                    #else:
-                    #    print(key, d, bonds[key])
+                        d = site0.distance(site1)
+
+                    key = "{:s}-{:s}".format(site1.specie.value, site0.specie.value)
+                    if d < bonds[key]:
+                        if pbc: site1.frac_coords += image
+                        sites_add.append(site1)
+                        ids_add.append(site1.index)
+                    
         if len(sites_add) > 0:
             visited.extend(sites_add)
 
@@ -476,6 +516,12 @@ def search_molecules_in_crystal(struc, tol=0.2, once=False, ignore_HH=True):
 
 if __name__ == "__main__":
 
+    from pyxtal.database.collection import Collection
+
     pmg = Structure.from_file('pyxtal/database/cifs/resorcinol.cif')
     mols = search_molecules_in_crystal(pmg, tol=0.2, once=False)
     print(len(mols))
+
+    pmg = Collection("molecules")['xxv']
+    mols = search_molecules_in_crystal(pmg, tol=0.2, once=False)
+    print(len(mols))
