@@ -28,7 +28,30 @@ class Error(Exception):
     """Base class for exceptions in this module."""
     pass
 
-class CompatibilityError(Error):
+class ConformerError(Error):
+    """Exception raised for errors in the Compabality.
+
+    Attributes:
+        expression -- input expression in which the error occurred
+        message -- explanation of the error
+    """
+
+    def __init__(self, message):
+        self.message = message
+
+class Symm_CompatibilityError(Error):
+    """Exception raised for errors in the Compabality.
+
+    Attributes:
+        expression -- input expression in which the error occurred
+        message -- explanation of the error
+    """
+
+    def __init__(self, message):
+        self.message = message
+
+
+class Comp_CompatibilityError(Error):
     """Exception raised for errors in the Compabality.
 
     Attributes:
