@@ -425,84 +425,6 @@ def get_inverse_ops(ops):
     return inverses
 
 
-def project_point(point, op, lattice=np.eye(3), PBC=[1, 1, 1]):
-    """
-    Given a 3-vector and a Wyckoff position operator, returns the projection of that
-    point onto the axis, plane, or point.
-
-    >>> from pyxtal.symmetry import Wyckoff_position
-    >>> ops1 = Wyckoff_position.from_group_and_index(14,1)
-    >>> ops0 = Wyckoff_position.from_group_and_index(14,0)
-    >>> print(ops)
-    Wyckoff position 2d in space group 14 with site symmetry -1
-    1/2, 0, 1/2
-    1/2, 1/2, 0
-    >>> project_point([0,0.3,0.1], ops0[0])
-    array([0. , 0.3, 0.1])
-    >>> project_point([0,0.3,0.1], ops1[0])
-    array([0.5, 0. , 0.5])
-
-    Args:
-        point: a 3-vector (numeric list, tuple, or array)
-        op: a SymmOp object representing a symmetry element within a symmetry group
-        lattice: 3x3 matrix describing the unit cell vectors
-        PBC: A periodic boundary condition list, 
-            where 1 means periodic, 0 means not periodic.
-            Ex: [1,1,1] -> full 3d periodicity, 
-                [0,0,1] -> periodicity along the z axis
-
-    Returns:
-        a transformed 3-vector (numpy array)
-    """
-    rot = op.rotation_matrix
-    trans = op.translation_vector
-
-    if PBC == [0, 0, 0]:
-        return project_point_no_PBC(point, rot, trans, lattice)
-    else:
-        # With PBC, the point could be projected onto multiple places on the symmetry element
-        point = filtered_coords(point)
-        # Generate translation vectors for the equivalent symmetry elements
-        m = create_matrix(PBC=PBC)
-        # Create new symmetry elements for each of the PBC vectors
-        new_vectors = []
-        distances = []
-        for v in m:
-            new_vector = project_point_no_PBC(point, rot, trans+v, lattice)
-            new_vectors.append(new_vector)
-            tmp = (new_vector-point).dot(lattice)
-            distances.append(np.linalg.norm(tmp))
-        i = np.argmin(distances)
-        return filtered_coords(new_vectors[i], PBC=PBC)
-
-
-def project_point_no_PBC(point, rot, trans, lattice=np.eye(3)):
-    """
-    Given a 3-vector and a Wyckoff position operator, returns the projection of that
-    point onto the axis, plane, or point.
-
-    Args:
-        point: a 3-vector (numeric list, tuple, or array)
-        op: a SymmOp object representing a symmetry element within a symmetry group
-        lattice: 3x3 matrix describing the unit cell vectors
-
-    Returns:
-        a transformed 3-vector (numpy array)
-    """
-    # Temporarily move the point in the opposite direction of the translation
-    point -= trans
-    new_vector = np.zeros(3)
-    # Loop over basis vectors of the symmetry element
-    # QZ: to optimize
-    for basis_vector in rot.T:
-        # b = np.linalg.norm(basis_vector)
-        b = np.sqrt(basis_vector.dot(basis_vector))  # a faster version?
-        #if not np.isclose(b, 0):
-        if b > 1e-3:
-            new_vector += basis_vector * (np.dot(point, basis_vector) / (b ** 2))
-    new_vector += trans
-    return new_vector
-
 def apply_ops(coord, ops):
     """
     Apply a list of SymmOps to a single 3-vector and return an array of
