@@ -13,8 +13,8 @@ from pymatgen.core.operations import SymmOp
 
 from pyxtal import pyxtal
 from pyxtal.lattice import Lattice
+from pyxtal.molecule import pyxtal_molecule
 from pyxtal.symmetry import Group, Wyckoff_position, get_wyckoffs
-from pyxtal.wyckoff_site import WP_merge
 from pyxtal.XRD import Similarity
 from pyxtal.operations import get_inverse
 
@@ -23,6 +23,7 @@ l0 = Lattice.from_matrix([[4.08, 0, 0], [0, 9.13, 0], [0, 0, 5.50]])
 l1 = Lattice.from_matrix([[4.08, 0, 0], [0, 9.13, 0], [0, 0, 5.50]])
 l2 = Lattice.from_para(4.08, 9.13, 5.50, 90, 90, 90)
 l3 = Lattice.from_para(4.08, 7.13, 5.50, 90, 38, 90, ltype="monoclinic")
+l4 = Lattice.from_para( 71.3649,   9.1273,  10.0753,  90.0000,  20.7978,  90.0000, ltype="monoclinic")
 wp1 = Wyckoff_position.from_group_and_index(36, 0)
 wp2 = Wyckoff_position.from_group_and_index(36, "4a")
 
@@ -82,9 +83,8 @@ class TestOptLat(unittest.TestCase):
         for diag in diags:
             for i in range(40):
                 sg = choice(sgs)
-                num = len(Group(sg)[0])
                 c1 = pyxtal(molecular=True)
-                c1.from_random(3, sg, ["aspirin"], [num], diag=diag) 
+                c1.from_random(3, sg, ["aspirin"], diag=diag) 
                 pmg1 = c1.to_pymatgen()
                 c2 = c1.copy()
                 c2.optimize_lattice(1)
@@ -95,9 +95,8 @@ class TestOptLat(unittest.TestCase):
         sgs, diag = [5, 7, 8, 12, 13, 14], False
         for i in range(40):
             sg = choice(sgs)
-            num = len(Group(sg)[0])
             c1 = pyxtal(molecular=True)
-            c1.from_random(3, sg, ["aspirin"], [num], diag=diag) 
+            c1.from_random(3, sg, ["aspirin"], diag=diag) 
             pmg1 = c1.to_pymatgen()
             c2 = c1.copy()
             c2.optimize_lattice(1)
@@ -113,10 +112,10 @@ class TestWP(unittest.TestCase):
         self.assertTrue(symbol == "4a")
 
     def test_merge(self):
-        pt, wp, _ = WP_merge([0.05, 0.7, 0.24], l1.get_matrix(), wp1, 0.5)
+        pt, wp, _ = wp1.merge([0.05, 0.7, 0.24], l1.get_matrix(), 0.5)
         symbol = str(wp.multiplicity) + wp.letter
         self.assertTrue(symbol == "4a")
-        pt, wp, _ = WP_merge([0.15, 0.7, 0.24], l1.get_matrix(), wp1, 0.5)
+        pt, wp, _ = wp1.merge([0.15, 0.7, 0.24], l1.get_matrix(), 0.5)
         symbol = str(wp.multiplicity) + wp.letter
         self.assertTrue(symbol == "8b")
 
@@ -125,7 +124,6 @@ class TestWP(unittest.TestCase):
             get_wyckoffs(i)
             get_wyckoffs(i, organized=True)
 
-    # to add test from string
 class TestDof(unittest.TestCase):
     def test_atomic(self):
         s = pyxtal() 
@@ -134,11 +132,18 @@ class TestDof(unittest.TestCase):
         self.assertTrue(s.lattice.dof == 1)
         self.assertTrue(ans == 1)
 
+class TestMolecule(unittest.TestCase):
+    def test_get_orientations_in_wp(self):
+        m = pyxtal_molecule('Benzene')
+        g = Group(61)
+        self.assertTrue(len(m.get_orientations_in_wp(g[0])) == 1)
+        self.assertTrue(len(m.get_orientations_in_wp(g[1])) == 1)
+        self.assertTrue(len(m.get_orientations_in_wp(g[2])) == 1)
+
 class TestMolecular(unittest.TestCase):
     def test_single_specie(self):
-        # print("test_h2o")
         struc = pyxtal(molecular=True)
-        struc.from_random(3, 36, ["H2O"], [8], sites=[["8b"]])
+        struc.from_random(3, 36, ["H2O"], sites=[["8b"]])
         struc.to_file()
         self.assertTrue(struc.valid)
 
@@ -147,7 +152,6 @@ class TestMolecular(unittest.TestCase):
         sga = SpacegroupAnalyzer(pmg_struc)
         # print(sga.get_space_group_symbol())
         self.assertTrue(sga.get_space_group_number() >= 36)
-        # print(pmg_struc.frac_coords[:3])
 
         # test rotation
         ax = struc.mol_sites[0].orientation.axis
@@ -160,7 +164,7 @@ class TestMolecular(unittest.TestCase):
 
     def test_sites(self):
         struc = pyxtal(molecular=True)
-        struc.from_random(3, 19, ["H2O"], [4])
+        struc.from_random(3, 19, ["H2O"])
         pmg_struc = struc.to_pymatgen()
         sga = SpacegroupAnalyzer(pmg_struc)
         self.assertTrue(sga.get_space_group_symbol() == "P2_12_12_1")
@@ -183,17 +187,11 @@ class TestMolecular(unittest.TestCase):
         pmg_s2 = C.to_pymatgen()
         self.assertTrue(sm.StructureMatcher().fit(pmg_struc, pmg_s2))
 
-    #def test_ice(self):
-    #    struc = pyxtal(molecular=True)
-    #    struc.from_seed(seed=cif_path+"ice.cif", molecule='H2O')
-    #    N = struc.check_short_distances_by_dict({"H-H": 1.0, "O-O": 2.0})
-    #    self.assertTrue(N==0)
-
     def test_big_molecule(self):
         # print("test_big_molecule")
         for mol in ["ROY", "aspirin"]:
             struc = pyxtal(molecular=True)
-            struc.from_random(3, 19, [mol], [4], 1.2)
+            struc.from_random(3, 19, [mol], factor=1.2)
             self.assertTrue(struc.valid)
             pair = struc.check_short_distances()
             if len(pair) > 0:
@@ -227,16 +225,13 @@ class TestMolecular(unittest.TestCase):
     def test_molecular_2d(self):
         # print("test_molecular_2d")
         struc = pyxtal(molecular=True)
-        struc.from_random(2, 20, ["H2O"], [4], 1.0, conventional=False)
-        cif = struc.to_file()
+        struc.from_random(2, 20, ["H2O"])
         self.assertTrue(struc.valid)
 
     def test_molecular_1d(self):
         struc = pyxtal(molecular=True)
-        struc.from_random(1, 20, ["H2O"], [4], 1.0, conventional=False)
-        cif = struc.to_file()
+        struc.from_random(1, 20, ["H2O"])
         self.assertTrue(struc.valid)
-        # def test_space_groups(self):
 
     def test_preassigned_sites(self):
         sites = [["4a", "4a"]]
@@ -244,6 +239,11 @@ class TestMolecular(unittest.TestCase):
         struc.from_random(3, 36, ["H2O"], [8], sites=sites)
         self.assertTrue(struc.valid)
 
+    def test_special_sites(self):
+        struc = pyxtal(molecular=True)
+        struc.from_random(3, 61, ["Benzene"], [4])
+        self.assertTrue(struc.valid)
+
 class TestAtomic3D(unittest.TestCase):
     def test_single_specie(self):
         struc = pyxtal()
@@ -333,8 +333,18 @@ class TestLattice(unittest.TestCase):
         self.assertTrue(abc, np.array([9.13, 4.08, 5.50]))
 
     def test_optimize(self):
-        l4, tran, _ = l3.optimize()
-        self.assertTrue(abs(l4.beta-1.495907)<1e-4)
+        lat, tran, _ = l3.optimize()
+        self.assertTrue(abs(lat.beta-1.495907)<1e-4)
+
+    def test_multi_optimize(self):
+        lat0 = l4
+        for i in range(10):
+            lat, tran, opt = lat0.optimize()
+            if opt:
+                lat0 = lat
+            else:
+                break
+        self.assertTrue(abs(lat.beta-1.7201)<1e-4)
 
     def test_setpara(self):
         l0.set_para([5, 5, 5, 90, 90, 90])
@@ -551,7 +561,5 @@ class Test_operations(unittest.TestCase):
         for i in range(1,229):
             res = Group(i).get_alternatives()['No.']
 
-# class TestIO(unittest.TestCase):
-
 if __name__ == "__main__":
     unittest.main()
