@@ -8,33 +8,226 @@ from scipy.spatial.transform import Rotation as R
 
 # External Libraries
 from pymatgen.core import Molecule
-from pymatgen.core.structure import Structure  
 
 # PyXtal imports
 from pyxtal.tolerance import Tol_matrix
 from pyxtal.operations import (
-    apply_ops, 
     check_images,
-    distance, 
     distance_matrix, 
-    project_point, 
     filtered_coords, 
     create_matrix,
     SymmOp,
 )
-from pyxtal.symmetry import Group, jk_from_i, Wyckoff_position
-from pyxtal.symmetry import ss_string_from_ops as site_symm
+from pyxtal.symmetry import Group, Wyckoff_position
 from pyxtal.database.element import Element
 from pyxtal.constants import rad, deg
 from pyxtal.lattice import Lattice
-from pyxtal.msg import printx
+
+class atom_site:
+    """
+    Class for storing atomic Wyckoff positions with a single coordinate.
+
+    Args:
+        wp: a `Wyckoff_position <pyxtal.symmetry.Wyckoff_position.html> object 
+        coordinate: a fractional 3-vector for the generating atom's coordinate
+        specie: an Element, element name or symbol, or atomic number of the atom
+        diag: whether or not has the diagonal symmetry
+        search: to search for the optimum position for special wyckoff site
+    """
+
+    def __init__(self, wp=None, coordinate=None, specie=1, diag=False, search=False):
+        self.position = np.array(coordinate)
+        self.specie = Element(specie).short_name
+        self.diag = diag
+        self.wp = wp
+        if self.diag:
+            self.wp.diagonalize_symops()
+            #self.position = project_point(self.position, wp[0])
+
+        self._get_dof()
+        self.PBC = self.wp.PBC
+        self.multiplicity = self.wp.multiplicity
+        if search:
+            self.search_position()
+        self.update()
+
+    def __str__(self):
+        if not hasattr(self.wp, "site_symm"): self.wp.get_site_symmetry()
+        s = "{:>2s} @ [{:7.4f} {:7.4f} {:7.4f}], ".format(self.specie, *self.position)
+        s += "WP [{:}{:}] ".format(self.wp.multiplicity, self.wp.letter)
+        s += "Site [{:}]".format(self.wp.site_symm.replace(" ",""))
+
+        return s
+
+    def __repr__(self):
+        return str(self)
+
+    def save_dict(self):
+        dict0 = {"position": self.position,
+                 "specie": self.specie,
+                 "number": self.wp.number,
+                 "dim": self.wp.dim,
+                 "index": self.wp.index,
+                 "PBC": self.wp.PBC,
+                }
+        return dict0
+
+    def _get_dof(self):
+        """
+        get the number of dof for the given structures:
+        """
+        freedom = np.trace(self.wp.ops[0].rotation_matrix) > 0
+        self.dof = len(freedom[freedom==True])
+
+
+    @classmethod
+    def load_dict(cls, dicts):
+        """
+        load the sites from a dictionary
+        """
+        g = dicts["number"]
+        index = dicts["index"]
+        dim = dicts["dim"]
+        PBC = dicts["PBC"]
+        position = dicts["position"]
+        specie = dicts["specie"]
+        wp = Wyckoff_position.from_group_and_index(g, index, dim, PBC)
+        return cls(wp, position, specie)
+
+    def perturbate(self, lattice, magnitude=0.1):
+        """
+        Random perturbation of the site
+        
+        Args:
+            lattice: lattice vectors
+            magnitude: the magnitude of displacement (default: 0.1 A)
+        """
+        dis = (np.random.sample(3) - 0.5).dot(lattice)
+        dis /= np.linalg.norm(dis)
+        dis *= magnitude
+        pos = self.position + dis.dot(np.linalg.inv(lattice))
+        self.update(pos)
+ 
+    def search_position(self):
+        """
+        Sometimes, the initial posiition is not the proper generator
+        Needs to find the proper generator
+        """
+        if self.wp.index > 0:
+            wp0 = Group(self.wp.number, self.wp.dim)[0]
+            pos = self.position
+            coords = wp0.apply_ops(pos)
+            for coord in coords:
+                ans = self.wp.ops[0].operate(coord)
+                diff = coord - ans
+                diff -= np.floor(diff)
+                if np.sum(diff**2)<1e-4:
+                    self.position = coord - np.floor(coord)
+                    break
+
+    def swap_axis(self, swap_id, shift=np.zeros(3)):
+        """
+        sometimes space groups like Pmm2 allows one to swap the a,b axes
+        to get an alternative representation
+        """
+        self.position += shift
+        self.position = self.position[swap_id]
+        self.position -= np.floor(self.position)
+        self.wp, _ = self.wp.swap_axis(swap_id)
+        self.site_symm = site_symm(
+            self.wp.symmetry_m[0], self.wp.number, dim=self.wp.dim
+        )
+        self.update()
+
+    def shift_by_swap(self, swap_id):
+        """
+        check if a shift is needed during swap
+        May occur for special WP in the I/A/B/C/F cases
+        e.g., in space group 71 (Immm), the permutation
+        4j(1/2,0,0.2) -> (0.2,0,1/2) -> 4f(0.7,1/2,0)
+        it requires a shift of (0.5,0.5,0.5)
+        """
+        wp, shift = self.wp.swap_axis(swap_id)
+        return shift
+
+    def equivalent_set(self, tran, indices):
+        """
+        Transform the wp to another equivalent set.
+        Needs to update both wp and positions
+
+        Args:
+            tran: affine matrix
+            indices: the list of transformed wps 
+        """
+        self.position = SymmOp(tran).operate(self.position)
+        self.position -= np.floor(self.position)
+        self.wp = self.wp.equivalent_set(indices[self.wp.index]) #update the wp index
+        self.site_symm = site_symm(
+            self.wp.symmetry_m[0], self.wp.number, dim=self.wp.dim
+        )
+        self.update()
+
+
+    def update(self, pos=None):
+        """
+        Used to generate coords from self.position
+        """
+        if pos is None:
+            pos = self.position
+        self.coords = self.wp.apply_ops(pos) 
+        self.position = self.coords[0]
+
+    def check_with_ws2(self, ws2, lattice, tm, same_group=True):
+        """
+        Given two Wyckoff sites, checks the inter-atomic distances between them.
+
+        Args:
+            ws2: a different Wyckoff_site object (will always return False if
+            two identical WS's are provided)
+            lattice: a 3x3 cell matrix
+            same_group: whether or not the two WS's are in the same structure.
+            Default value True reduces the calculation cost
+        Returns:
+            True if all distances are greater than the allowed tolerances.
+            False if any distance is smaller than the allowed tolerance
+        """
+        # Ensure the PBC values are valid
+        if self.PBC != ws2.PBC:
+            raise ValueError("PBC values do not match between Wyckoff sites")
+        # Get tolerance
+        tol = tm.get_tol(self.specie, ws2.specie)
+        # Symmetry shortcut method: check only some atoms
+        if same_group is True:
+            # We can either check one atom in WS1 against all WS2, or vice-versa
+            # Check which option is faster
+            if self.multiplicity > ws2.multiplicity:
+                coords1 = [self.coords[0]]
+                coords2 = ws2.coords
+            else:
+                coords1 = [ws2.coords[0]]
+                coords2 = self.coords
+            # Calculate distances
+            dm = distance_matrix(coords1, coords2, lattice, PBC=self.PBC)
+            # Check if any distances are less than the tolerance
+            if (dm < tol).any():
+                return False
+            else:
+                return True
+        # No symmetry method: check all atomic pairs
+        else:
+            dm = distance_matrix(ws1.coords, ws2.coords, lattice, PBC=ws1.PBC)
+            # Check if any distances are less than the tolerance
+            if (dm < tol).any():
+                return False
+            else:
+                return True
 
 class mol_site:
     """
     Class for storing molecular Wyckoff positions and orientations within
     the molecular_crystal class. Each mol_site object represenents an
-    entire Wyckoff position, not necessarily a single molecule. This is the
-    molecular version of Wyckoff_site
+    entire Wyckoff position, not necessarily a single molecule. 
+    This is the molecular version of Wyckoff_site
 
     Args:
         mol: a `pyxtal_molecule <pyxtal.molecule.pyxtal_molecule.html>`_ object
@@ -65,22 +258,22 @@ class mol_site:
 
         if self.diag:
             self.wp.diagonalize_symops()
-            self.position = project_point(self.position, wp[0])
+            self.position = self.wp.project(self.position)
 
     def __str__(self):
-
-        if not hasattr(self, "site_symm"):
-            self.site_symm = site_symm(
-                self.wp.symmetry_m[0], self.wp.number, dim=self.wp.dim
-            )
+        if not hasattr(self.wp, "site_symm"): self.wp.get_site_symmetry()
         self.angles = self.orientation.r.as_euler('zxy', degrees=True)
         formula = self.mol.formula.replace(" ","")
-        s = "{:} @ [{:7.4f} {:7.4f} {:7.4f}]  ".format(formula, *self.position)
-        s += "WP: {:2d}{:s}, ".format(self.wp.multiplicity, self.wp.letter)
-        s += "Site symmetry {:} ==> Euler: ".format(self.site_symm)
-        s += "{:6.2f} {:6.2f} {:6.2f}".format(*self.angles)
+        s = "{:12s} @ [{:7.4f} {:7.4f} {:7.4f}]  ".format(formula, *self.position)
+        s += "WP [{:d}{:s}] ".format(self.wp.multiplicity, self.wp.letter)
+        s += "Site [{:}]".format(self.wp.site_symm.replace(" ",""))
+        if len(self.molecule.mol) > 1:
+            s += " Euler [{:6.1f} {:6.1f} {:6.1f}]".format(*self.angles)
 
         return s
+    
+    def __repr__(self):
+        return str(self)
 
     def _get_dof(self):
         """
@@ -89,7 +282,6 @@ class mol_site:
         freedom = np.trace(self.wp.ops[0].rotation_matrix) > 0
         self.dof = len(freedom[freedom==True])
 
-
     def save_dict(self):
         dict0 = {"position": self.position,
                  "number": self.wp.number,
@@ -123,6 +315,7 @@ class mol_site:
         wp = Wyckoff_position.from_group_and_index(g, index, dim)
         diag = dicts["diag"]
         lattice = Lattice.from_matrix(dicts["lattice"], ltype=dicts["lattice_type"])
+
         return cls(mol, position, orientation, wp, lattice, diag)
 
     def encode(self):
@@ -130,12 +323,14 @@ class mol_site:
         transform dict to 1D vector
         [x, y, z, or1, or2, or3, rotor1, rotor2, .etc]
         """
-        xyz, _ = self._get_coords_and_species(absolute=True, first=True)
-        xyz -= self.molecule.get_center(xyz)
-        rotor = self.molecule.get_torsion_angles(xyz)
-        ori, _, reflect = self.molecule.get_orientation(xyz)
-        #print(self.molecule.mol)
-        return list(self.position) + list(ori) + rotor + [reflect]
+        if len(self.molecule.mol)>1: 
+            xyz, _ = self._get_coords_and_species(absolute=True, first=True)
+            #if len(xyz)==3: print("encode: \n", self.molecule.mol.cart_coords)
+            rotor = self.molecule.get_torsion_angles(xyz)
+            ori, _, reflect = self.molecule.get_orientation(xyz)
+            return list(self.position) + list(ori) + rotor + [reflect]
+        else:
+            return list(self.position) + [0]
         
     def to_1D_dicts(self):
         """
@@ -145,13 +340,10 @@ class mol_site:
         dict0 = {"smile": self.molecule.smile}
         dict0["rotor"] = self.molecule.get_torsion_angles(xyz)
         dict0["orientation"], dict0["rmsd"], dict0["reflect"] = self.molecule.get_orientation(xyz)
-        
         angs = dict0["rotor"] 
-        rdkit_mol = self.molecule.rdkit_mol(self.molecule.smile)
-        conf0 = rdkit_mol.GetConformer(0)
+        #rdkit_mol = self.molecule.rdkit_mol(self.molecule.smile)
+        #conf0 = rdkit_mol.GetConformer(0)
         #print(self.molecule.set_torsion_angles(conf0, angs))
-        #import sys; sys.exit()
-
         #print("save matrix"); print(self.orientation.r.as_matrix())
         #print("save angle"); print(self.orientation.r.as_euler('zxy', degrees=True))
         #print("angle"); print(dict0["orientation"])
@@ -171,34 +363,39 @@ class mol_site:
         from pyxtal.molecule import pyxtal_molecule, Orientation
 
         mol = pyxtal_molecule(mol=dicts['smile']+'.smi', fix=True)
-        rdkit_mol = mol.rdkit_mol(mol.smile)
-        conf = rdkit_mol.GetConformer(0)
-        #print("try")
-        #print(conf.GetPositions()[:3])
-        #print(dicts["rotor"])
-        if dicts['reflect']:
-            mol.set_torsion_angles(conf, dicts["rotor"], False)
-        #    print(mol.set_torsion_angles(conf, dicts["rotor"], True))
-        #    #import sys; sys.exit()
-        xyz = mol.set_torsion_angles(conf, dicts["rotor"], dicts['reflect'])
-        mol.reset_positions(xyz)
+        if len(mol.mol) > 1:
+            if len(dicts['smile']) > 1:
+                conf = mol.rdkit_mol().GetConformer(0)
+                if dicts['reflect']:
+                    mol.set_torsion_angles(conf, dicts["rotor"], False)
+                xyz = mol.set_torsion_angles(conf, dicts["rotor"], dicts['reflect'])
+            else:
+                # for H2O, use the standard one
+                xyz = np.array([[-0.00111384,  0.36313718,  0.        ],
+                                [-0.82498189, -0.18196256,  0.        ], 
+                                [ 0.82609573, -0.18117463,  0.        ]])
+ 
+            mol.reset_positions(xyz)
+            matrix = R.from_euler('zxy', dicts["orientation"], degrees=True).as_matrix()
+            orientation = Orientation(matrix)
+        else:
+            orientation = Orientation(np.eye(3))
+
         g = dicts["number"]
         index = dicts["index"]
         dim = dicts["dim"]
-        matrix = R.from_euler('zxy', dicts["orientation"], degrees=True).as_matrix()
-        orientation = Orientation(matrix)
-        #if dicts['reflect']:
-        #    print('load'); print(xyz[:3])
-        #    print("aaaaaaaaaaaaaa"); print(xyz[:3].dot(orientation.matrix.T))
-        #    print("matrix"); print(orientation.matrix)
         wp = Wyckoff_position.from_group_and_index(g, index, dim, dicts["PBC"])
         diag = dicts["diag"]
         lattice = Lattice.from_matrix(dicts["lattice"], ltype=dicts["lattice_type"])
         position = dicts["center"] #np.dot(dicts["center"], lattice.inv_matrix)
+        position, wp, _ = wp.merge(position, lattice.matrix, 0.01)
 
         return cls(mol, position, orientation, wp, lattice, diag)
 
     def show(self, id=None, **kwargs):
+        """
+        display WP on the notebook
+        """
         from pyxtal.viz import display_molecular_site
         return display_molecular_site(self, id, **kwargs)
 
@@ -207,15 +404,13 @@ class mol_site:
         Used to generate coords and species for get_coords_and_species
 
         Args:
-            absolute: whether or not to return absolute (Euclidean)
-                coordinates. If false, return relative coordinates instead
+            absolute: return absolute or relative coordinates 
             PBC: whether or not to add coordinates in neighboring unit cells, 
-                used for distance checking
             first: whether or not to extract the information from only the first site
             unitcell: whether or not to move the molecular center to the unit cell
 
         Returns:
-            atomic coords: a numpy array of fractional coordinates for the atoms in the site
+            atomic coords: a numpy array of atomic coordinates in the site
             species: a list of atomic species for the atomic coords
         """
         coord0 = self.mol.cart_coords.dot(self.orientation.matrix.T)  #
@@ -273,10 +468,8 @@ class mol_site:
         (with angle=0), and calculates the new positions.
 
         Args:
-            absolute: whether or not to return absolute (Euclidean)
-                coordinates. If false, return relative coordinates instead
-            PBC: whether or not to add coordinates in neighboring unit cells, used for
-                distance checking
+            absolute: return absolute or relative coordinates 
+            PBC: whether or not to add coordinates in neighboring unit cells
             unitcell: whether or not to move the molecular center to the unit cell
 
         Returns:
@@ -285,20 +478,6 @@ class mol_site:
         """
         return self._get_coords_and_species(absolute, PBC, unitcell=unitcell)
 
-    #def get_centers(self, absolute=False):
-    #    """
-    #    Get the coordinates for the center of mass 
-
-    #    Returns:
-    #        A numpy array of fractional 3-vectors
-    #    """
-    #    centers = apply_ops(self.position, self.wp.ops)
-    #    # centers1 = filtered_coords(centers0, self.PBC)
-    #    if not absolute:
-    #        return centers
-    #    else:
-    #        return np.dot(centers, self.lattice.matrix)
-
     def perturbate(self, lattice, trans=0.1, rot=5):
         """
         Random perturbation of the molecular site
@@ -325,7 +504,7 @@ class mol_site:
         if absolute:
             disp = disp.dot(self.lattice.inv_matrix)
         position = self.position + disp
-        self.position = project_point(position, self.wp[0])
+        self.position = self.wp.project(position)
 
     def rotate(self, ax_id=0, ax_vector=None, angle=180):
         """
@@ -341,13 +520,30 @@ class mol_site:
             ax = ax_vector/np.linalg.norm(ax_vector)
         else:
             xyz = self.mol.cart_coords.dot(p.as_matrix().T) 
-            ax = self.molecule.get_principle_axes(xyz).T[axis]
+            ax = self.molecule.get_principle_axes(xyz).T[ax_id]
 
         q = R.from_rotvec(ax*rad*angle)
         o = q*p
         self.orientation.r = o 
         self.orientation.matrix = o.as_matrix()
 
+    #def is_compatible_symmetry(self, tol=0.3):
+    #    """
+    #    Check if the molecular symmetry matches the site symmetry
+    #    """
+    #    mol = self.molecule.mol
+    #    if len(mol) == 1 or self.wp.index==0:
+    #        return True
+    #    else:
+    #        pga = PointGroupAnalyzer(mol, tol)
+    #        for op in self.wp.symmetry_m[0]:
+    #            if not pga.is_valid_op(op):
+    #                return False
+    #        return True
+
+    def is_valid_orientation(self):
+        pass
+
     def get_mol_object(self, id=0):
         """
         make the pymatgen molecule object
@@ -377,6 +573,18 @@ class mol_site:
         else:
             raise ValueError("id is greater than the number of molecules")
 
+    def show_molecule_in_box(self, id=0):
+        """
+        display molecule with box
+
+        Args:
+            id (int): molecular id
+        """
+        from pyxtal.viz import display_molecule
+        mol = self.get_mol_object(id)
+        cell, vertices, center = self.molecule.get_box_coordinates(mol.cart_coords)
+        return display_molecule(mol, center, cell)
+
     def update(self, coords, lattice=None, absolute=False, update_mol=True):
         """
         After the geometry relaxation, the returned atomic coordinates
@@ -403,7 +611,6 @@ class mol_site:
         if match:
             #position = np.mean(coords, axis=0).dot(self.lattice.inv_matrix)
             position = center.dot(self.lattice.inv_matrix)
-            #position -= np.floor(position)
             self.position = position - np.floor(position)
             if update_mol:
                 self.orientation = Orientation(np.eye(3))
@@ -447,7 +654,7 @@ class mol_site:
 
         pos = self.position
         wp0 = self.wp
-        pos0 = apply_ops(pos, wp0)
+        pos0 = wp0.apply_ops(pos)
 
         if len(wp0) == 2:
             if self.diag: # P21/n -> Pn
@@ -456,7 +663,7 @@ class mol_site:
                 wp1.diagonalize_symops()
                 axes = [[0,1,2],[2,1,0]]
                 for ax in axes:
-                    pos1 = apply_ops(pos[ax], wp1)
+                    pos1 = wp1.apply_ops(pos[ax])
                     diff = (pos1[:, ax] - pos0)[1]
                     diff -= np.floor(diff)
                     if len(diff[diff==0]) >= 2:
@@ -472,7 +679,7 @@ class mol_site:
                 for group in groups:
                     wp1 = Wyckoff_position.from_group_and_index(group, 0)
                     for ax in axes:
-                        pos1 = apply_ops(pos[ax], wp1)
+                        pos1 = wp1.apply_ops(pos[ax])
                         diff = (pos1[:, ax] - pos0)[1]
                         diff -= np.floor(diff)
                         if len(diff[diff==0]) >= 2:
@@ -670,293 +877,3 @@ class mol_site:
         return True
 
 
-class atom_site:
-    """
-    Class for storing atomic Wyckoff positions with a single coordinate.
-
-    Args:
-        wp: a `Wyckoff_position <pyxtal.symmetry.Wyckoff_position.html> object 
-        coordinate: a fractional 3-vector for the generating atom's coordinate
-        specie: an Element, element name or symbol, or atomic number of the atom
-        search: to search for the optimum position for special wyckoff site
-    """
-
-    def __init__(self, wp=None, coordinate=None, specie=1, diag=False, search=False):
-        self.position = np.array(coordinate)
-        self.specie = Element(specie).short_name
-        self.diag = diag
-        self.wp = wp
-        if self.diag:
-            self.wp.diagonalize_symops()
-            #self.position = project_point(self.position, wp[0])
-
-        self._get_dof()
-        self.PBC = self.wp.PBC
-        self.multiplicity = self.wp.multiplicity
-        if search:
-            self.search_position()
-        self.update()
-
-    def __str__(self):
-        #number, dim = self.wp.number, self.wp.dim
-        #self.site_symm = site_symm(self.wp.symmetry_m[0], number, dim=dim)
-
-        if not hasattr(self, "site_symm"):
-            self.site_symm = site_symm(
-                self.wp.symmetry_m[0], self.wp.number, dim=self.wp.dim
-            )
-        s = "{:>2s} @ [{:7.4f} {:7.4f} {:7.4f}], ".format(self.specie, *self.position)
-        s += "WP: {:2d}{:s}, ".format(self.wp.multiplicity, self.wp.letter)
-        s += "Site symmetry: {:s}".format(self.site_symm)
-        return s
-
-    def __repr__(self):
-        return str(self)
-
-    def save_dict(self):
-        dict0 = {"position": self.position,
-                 "specie": self.specie,
-                 "number": self.wp.number,
-                 "dim": self.wp.dim,
-                 "index": self.wp.index,
-                 "PBC": self.wp.PBC,
-                }
-        return dict0
-
-    def _get_dof(self):
-        """
-        get the number of dof for the given structures:
-        """
-        freedom = np.trace(self.wp.ops[0].rotation_matrix) > 0
-        self.dof = len(freedom[freedom==True])
-
-
-    @classmethod
-    def load_dict(cls, dicts):
-        """
-        load the sites from a dictionary
-        """
-        g = dicts["number"]
-        index = dicts["index"]
-        dim = dicts["dim"]
-        PBC = dicts["PBC"]
-        position = dicts["position"]
-        specie = dicts["specie"]
-        wp = Wyckoff_position.from_group_and_index(g, index, dim, PBC)
-        return cls(wp, position, specie)
-
-    def perturbate(self, lattice, magnitude=0.1):
-        """
-        Random perturbation of the site
-        
-        Args:
-            lattice: lattice vectors
-            magnitude: the magnitude of displacement (default: 0.1 A)
-        """
-        dis = (np.random.sample(3) - 0.5).dot(lattice)
-        dis /= np.linalg.norm(dis)
-        dis *= magnitude
-        pos = self.position + dis.dot(np.linalg.inv(lattice))
-        self.update(pos)
- 
-    def search_position(self):
-        """
-        Sometimes, the initial posiition is not the proper generator
-        Needs to find the proper generator
-        """
-        if self.wp.index > 0:
-            pos = self.position
-            coords = apply_ops(pos, Group(self.wp.number, self.wp.dim)[0])
-            for coord in coords:
-                ans = apply_ops(coord, [self.wp.ops[0]])[0]
-                diff = coord - ans
-                diff -= np.floor(diff)
-                if np.sum(diff**2)<1e-4:
-                    self.position = coord - np.floor(coord)
-                    break
-    def swap_axis(self, swap_id, shift=np.zeros(3)):
-        """
-        sometimes space groups like Pmm2 allows one to swap the a,b axes
-        to get an alternative representation
-        """
-        self.position += shift
-        self.position = self.position[swap_id]
-        self.position -= np.floor(self.position)
-        self.wp, _ = self.wp.swap_axis(swap_id)
-        self.site_symm = site_symm(
-            self.wp.symmetry_m[0], self.wp.number, dim=self.wp.dim
-        )
-        self.update()
-
-    def shift_by_swap(self, swap_id):
-        """
-        check if a shift is needed during swap
-        May occur for special WP in the I/A/B/C/F cases
-        e.g., in space group 71 (Immm), the permutation
-        4j(1/2,0,0.2) -> (0.2,0,1/2) -> 4f(0.7,1/2,0)
-        it requires a shift of (0.5,0.5,0.5)
-        """
-        wp, shift = self.wp.swap_axis(swap_id)
-        return shift
-
-    def equivalent_set(self, tran, indices):
-        """
-        Transform the wp to another equivalent set.
-        Needs to update both wp and positions
-
-        Args:
-            tran: affine matrix
-            indices: the list of transformed wps 
-        """
-        self.position = SymmOp(tran).operate(self.position)
-        self.position -= np.floor(self.position)
-        self.wp = self.wp.equivalent_set(indices[self.wp.index]) #update the wp index
-        self.site_symm = site_symm(
-            self.wp.symmetry_m[0], self.wp.number, dim=self.wp.dim
-        )
-        self.update()
-
-
-    def update(self, pos=None):
-        """
-        Used to generate coords from self.position
-        """
-        if pos is None:
-            pos = self.position
-        self.coords = apply_ops(pos, self.wp) 
-        self.position = self.coords[0]
-
-    def check_with_ws2(self, ws2, lattice, tm, same_group=True):
-        """
-        Given two Wyckoff sites, checks the inter-atomic distances between them.
-
-        Args:
-            ws2: a different Wyckoff_site object (will always return False if
-            two identical WS's are provided)
-            lattice: a 3x3 cell matrix
-            same_group: whether or not the two WS's are in the same structure.
-            Default value True reduces the calculation cost
-        Returns:
-            True if all distances are greater than the allowed tolerances.
-            False if any distance is smaller than the allowed tolerance
-        """
-        # Ensure the PBC values are valid
-        if self.PBC != ws2.PBC:
-            printx("Error: PBC values do not match between Wyckoff sites")
-            return
-        # Get tolerance
-        tol = tm.get_tol(self.specie, ws2.specie)
-        # Symmetry shortcut method: check only some atoms
-        if same_group is True:
-            # We can either check one atom in WS1 against all WS2, or vice-versa
-            # Check which option is faster
-            if self.multiplicity > ws2.multiplicity:
-                coords1 = [self.coords[0]]
-                coords2 = ws2.coords
-            else:
-                coords1 = [ws2.coords[0]]
-                coords2 = self.coords
-            # Calculate distances
-            dm = distance_matrix(coords1, coords2, lattice, PBC=self.PBC)
-            # Check if any distances are less than the tolerance
-            if (dm < tol).any():
-                return False
-            else:
-                return True
-        # No symmetry method: check all atomic pairs
-        else:
-            dm = distance_matrix(ws1.coords, ws2.coords, lattice, PBC=ws1.PBC)
-            # Check if any distances are less than the tolerance
-            if (dm < tol).any():
-                return False
-            else:
-                return True
-
-def WP_merge(pt, lattice, wp, tol, orientations=None):
-    """
-    Given a list of fractional coordinates, merges them within a given
-    tolerance, and checks if the merged coordinates satisfy a Wyckoff
-    position. Used for merging general Wyckoff positions into special Wyckoff
-    positions within the random_crystal (and its derivative) classes.
-
-    Args:
-        pt: the originl point (3-vector)
-        lattice: a 3x3 matrix representing the unit cell
-        wp: a `Wyckoff_position <pyxtal.symmetry.Wyckoff_position.html> object after merge
-        tol: the cutoff distance for merging coordinates
-        orientations: the valid orientations for a given molecule. Obtained
-            from get_sg_orientations, which is called within molecular_crystal
-
-    Returns:
-        pt: 3-vector after merge
-        wp: a `pyxtal.symmetry.Wyckoff_position` object, If no matching WP, returns False. 
-        valid_ori: the valid orientations after merge
-
-    """
-    index = wp.index
-    PBC = wp.PBC
-    group = Group(wp.number, wp.dim)
-    pt = project_point(pt, wp[0], lattice, PBC)
-    coor = apply_ops(pt, wp)
-    if orientations is None:
-        valid_ori = None
-    else:
-        j, k = jk_from_i(index, orientations)
-        valid_ori = orientations[j][k]
-    
-    # Main loop for merging multiple times
-
-    while True:
-        # Check distances of current WP. If too small, merge
-        dm = distance_matrix([coor[0]], coor, lattice, PBC=PBC)
-        passed_distance_check = True
-        x = np.argwhere(dm < tol)
-        for y in x:
-            # Ignore distance from atom to itself
-            if y[0] == 0 and y[1] == 0:
-                pass
-            else:
-                passed_distance_check = False
-                break
-
-        # for molecular crystal, one more check
-        if not check_images([coor[0]], [6], lattice, PBC=PBC, tol=tol):
-            passed_distance_check = False
-
-        if not passed_distance_check:
-            mult1 = group[index].multiplicity
-            # Find possible wp's to merge into
-            possible = []
-            for i, wp0 in enumerate(group):
-                mult2 = wp0.multiplicity
-                # Check that a valid orientation exists
-                if orientations is not None:
-                    j, k = jk_from_i(i, orientations)
-                    if orientations[j][k] == []:
-                        continue
-                    else:
-                        valid_ori = orientations[j][k]
-                # factor = mult2 / mult1
-
-                if (mult2 < mult1) and (mult1 % mult2 == 0):
-                    possible.append(i)
-            if possible == []:
-                return None, False, valid_ori
-            # Calculate minimum separation for each WP
-            distances = []
-            for i in possible:
-                wp = group[i]
-                projected_point = project_point(pt.copy(), wp[0], lattice=lattice, PBC=PBC)
-                d = distance(pt - projected_point, lattice, PBC=PBC)
-                distances.append(np.min(d))
-            # Choose wp with shortest translation for generating point
-            tmpindex = np.argmin(distances)
-            index = possible[tmpindex]
-            wp = group[index]
-            pt = project_point(pt, wp[0], lattice=lattice, PBC=PBC)
-            coor = apply_ops(pt, wp)
-        # Distances were not too small; return True
-        else:
-            return pt, wp, valid_ori
-
-
